@@ -2,9 +2,12 @@ package Crypt::LE::Challenge::Simple;
 use strict;
 use warnings;
 
+our $VERSION = '0.05';
+
 =head1 NAME
 
-Crypt::LE::Challenge::Simple - A boilerplate for extending Crypt::LE with challenge handlers.
+Crypt::LE::Challenge::Simple - A boilerplate for extending Crypt::LE and Crypt::LE client
+application (le.pl) with challenge handlers.
 
 =head1 SYNOPSIS
 
@@ -21,6 +24,10 @@ Crypt::LE::Challenge::Simple - A boilerplate for extending Crypt::LE with challe
 Crypt::LE provides the functionality necessary to use Let's Encrypt API and generate free SSL certificates for your domains.
 This Crypt::LE plugin is an example of how challenge handling can be done by an external module.
 
+B<This module can also be used with the provided Crypt::LE client application - le.pl:>
+
+ le.pl ... --handle-with Crypt::LE::Challenge::Simple --handle-params '{"key1": 1, "key2": 2, "key3": "something"}'
+
 =cut
 
 sub new { bless {}, shift }
