@@ -1,7 +1,6 @@
 #!/usr/bin/env perl
 use strict;
 use warnings;
-use lib '../lib';
 use Getopt::Long;
 use IO::File;
 use JSON::MaybeXS;
@@ -10,13 +9,15 @@ use Crypt::LE ':errors';
 sub info { print shift, "\n" }
 sub bail { &info; exit 255 }
 
-my $VERSION = '0.04';
+my $VERSION = '0.05';
 
-my ($account_key, $csr, $csr_key, $domains, $challenge_path, $crt, $live, $generate_missing, $generate_only, $handle_with, $handle_as, $handle_params, $verified, $debug, $help);
+my ($account_key, $csr, $csr_key, $domains, $challenge_path, $crt, $live, $generate_missing, $generate_only, 
+    $handle_with, $handle_as, $handle_params, $complete_with, $complete_params, $verified, $debug, $help);
 
 GetOptions ("key=s" => \$account_key, "csr=s" => \$csr, "csr-key=s" => \$csr_key, "domains=s" => \$domains, "path=s" => \$challenge_path,
             "crt=s" => \$crt, "generate-missing" => \$generate_missing, "generate-only" => \$generate_only, "handle-with=s" => \$handle_with,
-	    "handle-as=s" => \$handle_as, "handle-params=s" => \$handle_params, "live" => \$live, "debug" => \$debug, "verified" => \$verified, "help" => \$help);
+	    "handle-as=s" => \$handle_as, "handle-params=s" => \$handle_params, "complete-with=s" => \$complete_with, 
+            "complete-params=s" => \$complete_params, "live" => \$live, "debug" => \$debug, "verified" => \$verified, "help" => \$help);
 
 my $usage = <<EOF;
 
@@ -27,21 +28,26 @@ Note: by default a staging server is used, which does not provide trusted certif
 
 Available parameters:
 
- key <file>                    - Your account key file.
- csr <file>                    - Your CSR file.
- csr-key <file>                - Key for your CSR (only mandatory if CSR is missing and to be generated).
- domains <list>                - Domains as comma-separated list (only mandatory if CSR is missing).
- path <absolute path>          - Path to local .well-known/acme-challenge/ to drop required challenge files into (optional).
- handle-with <Some::Module>    - Module name to handle challenges with (optional).
- handle-as <http|dns|tls|...>  - Type of challenge to request, by default 'http' (optional).
- handle-params <{json}>        - JSON with parameters to be passed to the challenge-handling module (optional).
- generate-missing              - Generate missing files (key, csr and csr-key).
- generate-only                 - Generate a new key and/or CSR if they are missing and then exit.
- crt <file>                    - Name for the domain certificate file.
- verified                      - Skip challenge verification steps (request/accept/verify). Use only if domains have been already verified recently.
- live                          - Connect to a live server instead of staging.
- debug                         - Print out debug messages.
- help                          - This screen.
+ key <file>                       - Your account key file.
+ csr <file>                       - Your CSR file.
+ csr-key <file>                   - Key for your CSR (only mandatory if CSR is missing and to be generated).
+ domains <list>                   - Domains as comma-separated list (only mandatory if CSR is missing).
+ path <absolute path>             - Path to local .well-known/acme-challenge/ to drop required challenge files into (optional).
+ handle-with <Some::Module>       - Module name to handle challenges with (optional).
+ handle-as <http|dns|tls|...>     - Type of challenge to request, by default 'http' (optional).
+ handle-params <{json}>           - JSON with parameters to be passed to the challenge-handling module (optional).
+ complete-with <Another::Module>  - Module name to handle process completion with (optional).
+ complete-params <{json}>         - JSON with parameters to be passed to the completion-handling module (optional).
+ generate-missing                 - Generate missing files (key, csr and csr-key).
+ generate-only                    - Generate a new key and/or CSR if they are missing and then exit.
+ crt <file>                       - Name for the domain certificate file.
+ verified                         - Skip challenge verification steps (request/accept/verify). Use only if domains have been already verified recently.
+ live                             - Connect to a live server instead of staging.
+ debug                            - Print out debug messages.
+ help                             - This screen.
+
+[ See Crypt::LE::Challenge::Simple for an example of challenge handling module. ]
+[ See Crypt::LE::Complete::Simple for an example of completion handling module. ]
 
 EOF
 
@@ -67,7 +73,7 @@ if ($challenge_path) {
     bail "Path to save challenge files into should be a writable directory" unless (-d $challenge_path and -w _);
 }
 
-my ($handler, $params);
+my ($handler, $complete_handler);
 $handle_as = $handle_as ? lc($handle_as) : 'http';
 
 if ($handle_with) {
@@ -79,11 +85,30 @@ if ($handle_with) {
     if ($handle_params) {
 	my $j = JSON->new->canonical()->allow_nonref();
         eval {
-	    $params = $j->decode($handle_params);
+	    $handle_params = $j->decode($handle_params);
         };
         bail "Could not decode parameters for the challenge-handling module: $@" if $@;
-        bail "Please make sure that your JSON document has {} around it." unless (ref $params eq 'HASH');
+        bail "Please make sure that your 'handle-params' JSON document has {} around it." unless (ref $handle_params eq 'HASH');
     }
+} else {
+    undef $handle_params;
+}
+
+if ($complete_with) {
+    eval "use $complete_with;";
+    bail "Cannot use the module to complete processing with." if $@;
+    $complete_handler = new $complete_with;
+    bail "Module to complete processing with does not seem to support the required 'complete' method." unless $complete_handler->can('complete');
+    if ($complete_params) {
+        my $j = JSON->new->canonical()->allow_nonref();
+        eval {
+            $complete_params = $j->decode($complete_params);
+        };
+        bail "Could not decode parameters for the completion module: $@" if $@;
+        bail "Please make sure that your 'complete-params' JSON document has {} around it." unless (ref $complete_params eq 'HASH');
+    }
+} else {
+    undef $complete_params;
 }
 
 # Begin work
@@ -118,7 +143,7 @@ bail $le->error_details if $le->register;
 $le->accept_tos(); # No need to check explicitly for tos_changed() - Crypt::LE will handle that for you.
 unless ($verified) {
     bail $le->error_details if $le->request_challenge();
-    bail $le->error_details if $le->accept_challenge($handler ? $handler : \&process_challenge, $params, $handle_as);
+    bail $le->error_details if $le->accept_challenge($handler ? $handler : \&process_challenge, $handle_params, $handle_as);
     bail $le->error_details if $le->verify_challenge;
 }
 info "Requesting domain certificate";
@@ -132,6 +157,20 @@ if ($le->request_issuer_certificate()) {
     info "Saving the full certificate chain (With the proper server configuration and HSTS you can get an 'A+' on Qualys SSL test)";
     bail "Failed to save the domain certificate file" if _write($crt, $le->certificate . "\n" . $le->issuer);
 }
+if ($complete_handler) {
+    my $data = {
+        # Note, certificate here is just a domain certificate, issuer is passed separately - so handler 
+        # could merge those or use them separately as well.
+        certificate => $le->certificate, certificate_file => $crt, key_file => $csr_key, issuer => $le->issuer,
+    };
+    my $rv;
+    eval {
+	$rv = $complete_handler->complete($data, $complete_params);
+    };
+    if ($@ or !$rv) {
+        bail "Completion handler " . ($@ ? "thrown an error: $@" : "did not return a true value");
+    }
+}
 
 info "The job is done, enjoy your certificate! For feedback and bug reports contact me at [ https://do-know.com ]";
 
